#include <code/json/marshaling/mapping.hxx>
#include <code/json/marshaling/serialize.hxx>

#include <iostream>

#define DEFINE_TEST(x) std::cout << x << '\n';
#define TEST_TRUE(x) if (!(x)) return __LINE__;
#define TEST_FALSE(x) if ((x)) return __LINE__;
#define TEST_EQUAL(x, y) if ((x) != (y)) return __LINE__;

struct person_name {
  std::string first;
  std::string last;

  static code::json::marshaling::mapping< person_name > const&
  json()
  {
    static code::json::marshaling::mapping< person_name > const mapping{
      code::json::marshaling::member("first", &person_name::first),
      code::json::marshaling::member("last", &person_name::last)
    };

    return mapping;
  }
};

struct person {
  person_name name;
  int age;

  static code::json::marshaling::mapping< person > const&
  json()
  {
    static code::json::marshaling::mapping< person > const mapping{
      code::json::marshaling::member("person", &person::name),
      code::json::marshaling::member("age", &person::age)
    };

    return mapping;
  }
};

int
main()
{
  DEFINE_TEST("round trip")
  {
    using namespace code::json::marshaling;

    person p1{ { "Jane", "Doe" }, 37 };

    auto p2 = deserialize< person >(serialize(p1));

    TEST_EQUAL(p1.name.first, p2.name.first);
    TEST_EQUAL(p1.name.last, p2.name.last);
    TEST_EQUAL(p1.age, p2.age);
  }

  return 0;
}
